/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

#define INTERVAL  (0.5 * 1000)  /* timeout */
#define PIE       (60 * 64)     /* angle of pie */
#define ROTATION  (20 * 64)     /* angle of the rotation */

static void TimeoutCB();
static Widget canvas;  /* drawing area widget on which pie is drawn */
static XtIntervalId timeout_id;  /* the ID of timeout callback */
static GC gc;  /* Graphics Context necessary for Xlib drawing */
static int angle = (90 * 64) - PIE;  /* initial angle of pie */

/* Called when "Start" button is pressed */
static void StartCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget start = (Widget)w, stop  = (Widget)client_data;

    /* Register timeout callback */
    timeout_id = XtAppAddTimeOut(XtWidgetToApplicationContext(canvas),
                                 INTERVAL, TimeoutCB, NULL);
    XtSetSensitive(start, False);  /* Disable "Start" button */
    XtSetSensitive(stop, True);    /* Enable "Stop" button */
}

/* Timeout callback */
static void TimeoutCB(client_data, id)
    XtPointer     client_data;  /* Client data passed at registration */
    XtIntervalId *id;           /* The ID of timeout callback */
{
    XClearWindow(XtDisplay(canvas), XtWindow(canvas)); /* clear the window */
    XFillArc(XtDisplay(canvas), XtWindow(canvas), gc,  /* draw a pie */
             50, 50, 100, 100, angle, PIE);
    /* This timeout callback should be called again */
    timeout_id = XtAppAddTimeOut(XtWidgetToApplicationContext(canvas),
                                 INTERVAL, TimeoutCB, NULL);
    angle -= ROTATION;  /* calculate the starting angle of the next pie */
    if (angle < 0) angle += (360 * 64);
}

/* Called when "Stop" button is pressed */
static void StopCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget start = (Widget)client_data, stop  = (Widget)w;

    /* Delete the timeout callback */
    XtRemoveTimeOut(timeout_id);

    XtSetSensitive(start, True);  /* Enable "Start" button */
    XtSetSensitive(stop, False);  /* Disable "Stop" button */
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, panel, bboard, control, start, stop;
    Arg    al[20];
    int    ac;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Timeoutcb", NULL, 0,
                               &argc, argv, NULL, al, ac);
    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);

    ac = 0;
    bboard = XmCreateBulletinBoard(panel, "bboard", al, ac);
    XtManageChild(bboard);

    ac = 0;
    canvas = XmCreateDrawingArea(bboard, "canvas", al, ac);
    XtManageChild(canvas);

    ac = 0;
    control = XmCreateRowColumn(panel, "control", al, ac);
    XtManageChild(control);

    ac = 0;
    start = XmCreatePushButton(control, "start", al, ac);
    XtManageChild(start);

    ac = 0;
    /* Initially, "Stop" button cannot be used */
    XtSetArg(al[ac], XmNsensitive, False); ac++;
    stop = XmCreatePushButton(control, "stop", al, ac);
    XtManageChild(stop);

    XtAddCallback(start, XmNactivateCallback, StartCB, stop);
    XtAddCallback(stop, XmNactivateCallback, StopCB, start);

    XtRealizeWidget(toplevel);

    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas), NULL, NULL);
    XSetForeground(XtDisplay(canvas), gc, BlackPixelOfScreen(XtScreen(canvas)));

    XtAppMainLoop(app_context);
}
